/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoAttribut;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3InfoElement;

/**
 * TIC3-Wiederholung.
 * <p>
 * Entspricht TIC3-Typ 'RecurrenceType'.
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3Wiederholung.java 63650 2016-09-22 17:11:32Z gieseler $
 */
public class TIC3Wiederholung extends AbstractTIC3InfoElement {
	/**
	 * TIC3 Info XML Element: Recurrence (REC), A list of recurring periods
	 * between the overall begin and end times during which the overall schedule
	 * is limited to specific days and times.
	 */
	public static final String XML_ELEMENT_WIEDERHOLUNG = "Recurrence";

	/**
	 * DAV-Attribut.
	 */
	public static final String DAV_ATTR_WIEDERHOLUNG = "Wiederholungen";

	/**
	 * Beschreibung.
	 * <p>
	 * Beschreibung aus Wert des 'description'-Attributes.
	 * </p>
	 */
	private TIC3InfoAttribut beschreibung = new TIC3InfoAttribut("description", "Beschreibung");

	/**
	 * Startzeit.
	 * <p>
	 * TIC3 Info XML Element: StartTime (STA), The start time on each of the
	 * specified days of week.
	 * </p>
	 */
	private final TIC3Text startzeit = new TIC3Text("StartTime", "Startzeit");

	/**
	 * Endzeit.
	 * <p>
	 * TIC3 Info XML Element: EndTime (END), The end time on each of the
	 * specified days of week. When the end time is less than the start time,
	 * the end time is on the next day.
	 * </p>
	 */
	private final TIC3Text endZeit = new TIC3Text("EndTime", "Endzeit");

	/**
	 * Wochentage.
	 * <p>
	 * TIC3 Info XML Element: DayOfWeek (DOW), A list of days of the week such
	 * as Monday and Tuesday.
	 * </p>
	 */
	private final List<TIC3Wochentag> wochentage = new ArrayList<>();

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3Wiederholung(final String xmlName, final String davName) {
		super(xmlName, davName);
		addAttribut(beschreibung);
		addKindElement(startzeit);
		addKindElement(endZeit);
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);

		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);
			if (n.getNodeName().equals(TIC3Wochentag.XML_ELEMENT_WEEKDAY)) {
				final TIC3Wochentag loc = new TIC3Wochentag(TIC3Wochentag.XML_ELEMENT_WEEKDAY,
						TIC3Wochentag.DAV_ATTR_WEEKDAY);
				loc.einlesenXML(n);
				wochentage.add(loc);
			}
		}
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		for (final TIC3InfoElement child : getKindElemente()) {
			child.publizieren(davdata, parameter);
		}

		for (final TIC3InfoElement child : getAttribute()) {
			child.publizieren(davdata, parameter);
		}

		// Wochentage
		if (wochentage.size() > 0) {
			final Data.Array array = davdata.getArray(TIC3Wochentag.DAV_ATTR_WEEKDAY);

			array.setLength(wochentage.size());
			for (int i = 0; i < wochentage.size(); i++) {
				wochentage.get(i).publizieren(array.getItem(i), parameter);
			}
		}
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName() + ": Beschreibung='" + beschreibung.getXmlValue();
	}
}
